% this program simulates the sinusoidally forced Duffing oscillator
% for some time T, for a given initial condition
% the hope is to change the forcing frequency omega and the forcing
% amplitude A and see what the relation of the oscillation amplitude is
% to the forcing

% starting with a clean slate
clc; clear all; close all;

%% equation of motion is
% xdotdot + + epsilon*c*xdot + k*x + epsilon*alpha*x^3  = A*cos(omega*t)
% param.c = 0.05; param.k = 1; param.epsilon = 0.5;

% % Use for Hardening. GOOD DEFAULT
% param.c = 0.5; param.k = 1; 
% param.alpha = 0.5;  % cubic nonlinearity
% param.epsilon = 0.1; % small parameter that keeps damping and nonlinearity small
% param.A = 1; % forcing amplitude DEFAULT
% omegaMax = 3.5; omegaMin = 0.1; % forcing frequency range

% % Use for Softening
% param.c = 0.1; param.k = 1; 
% param.alpha = -3;  % cubic nonlinearity
% param.epsilon = 0.1; % small parameter that keeps damping and nonlinearity small
% param.A = 0.01; % forcing amplitude DEFAULT
% omegaMax = 2; omegaMin = 0.1; % forcing frequency range

% alternate softening
param.c = 0.2; param.k = 1; 
param.alpha = -4;  % cubic nonlinearity, softening
param.epsilon = 0.1; % small parameter that keeps damping and nonlinearity small
param.A = 0.02; % forcing amplitude
omegaMax = 3.5; omegaMin = 0.1; % forcing frequency range



% solving the ODE options
options = odeset('reltol',1e-6,'abstol',1e-6);

%% forward sweep of frequency
numFrequenciesUsed = 80;
omegalist_fwd = linspace(omegaMin,omegaMax,numFrequenciesUsed)';
OscAmpl_list_fwd = zeros(size(omegalist_fwd));

% initial condition
Y0 = [0.01; 0];

for i = 1:length(omegalist_fwd)
    
    i
    
    param.omega = omegalist_fwd(i);
    
    %% time duration and time list
    numCyclesOfSimulation = 50;
    tmax = numCyclesOfSimulation*(2*pi/param.omega); % need to be long enough for near steady state, given low damping (lower the damping longer this needs to be)
    numPointsPerCycle = 50;
    tspan = linspace(0,tmax,numPointsPerCycle*numCyclesOfSimulation)'; % equally spaced time output, densse enough to capture oscillation peaks well enough
    
    
    [tlist,Ylist] = ode45(@OdeForcedDuffing,tspan,Y0,options,param);
    OscAmpl_list_fwd(i) = (max(Ylist(end-5*numPointsPerCycle:end,1))-min(Ylist(end-5*numPointsPerCycle:end,1)))/2; % steady state amplitude
    
    OscAmpl_list_fwd(i) = OscAmpl_list_fwd(i)/(param.A/param.k);
    
    %
    %     figure(2); % hold on;
    %     plot(tlist,Ylist(:,1));
    %     pause
    %
    % resetting initial condition
    Y0 = Ylist(end,:)';
    
    %     figure(1);
    %     plot(omegalist_fwd(i),OscAmpl_list_fwd(i),'mx'); hold on;
    %     xlabel('Forcing frequency \omega');
    %     ylabel('Steady Oscillation Amplitude');
    
    
    
end

%% backward sweep of frequency
omegalist_bckwd = omegalist_fwd(end:-1:1);
OscAmpl_list_bckwd = zeros(size(omegalist_bckwd));

for i = 1:length(omegalist_bckwd)
    
    i
    
    param.omega = omegalist_bckwd(i);
    %% time duration and time list
    numCyclesOfSimulation = 50;
    tmax = numCyclesOfSimulation*(2*pi/param.omega); % need to be long enough for near steady state, given low damping (lower the damping longer this needs to be)
    numPointsPerCycle = 50;
    tspan = linspace(0,tmax,numPointsPerCycle*numCyclesOfSimulation)'; % equally spaced time output, densse enough to capture oscillation peaks well enough
    
    
    [tlist,Ylist] = ode45(@OdeForcedDuffing,tspan,Y0,options,param);
    OscAmpl_list_bckwd(i) = (max(Ylist(end-5*numPointsPerCycle:end,1))-min(Ylist(end-5*numPointsPerCycle:end,1)))/2; % steady state amplitude
    OscAmpl_list_bckwd(i) = OscAmpl_list_bckwd(i)/(param.A/param.k);
    
    % resetting initial condition
    Y0 = Ylist(end,:)';
    
    %     figure(1);
    %     plot(omegalist_bckwd(i),OscAmpl_list_bckwd(i),'bo'); hold on;
    %     xlabel('Forcing frequency \omega');
    %     ylabel('Steady Oscillation Amplitude')
    %     legend('forward frequency sweep', 'backward frequency sweep');
    
end

figure(1);
plot(omegalist_fwd,OscAmpl_list_fwd,'mx'); hold on;
plot(omegalist_bckwd,OscAmpl_list_bckwd,'bo','markersize',6);
xlabel('Forcing frequency \omega');
ylabel('Steady Oscillation Amplitude')
legend('forward frequency sweep', 'backward frequency sweep');

%%
figure(2);
plot(omegalist_fwd,OscAmpl_list_fwd,'mx--'); hold on;
plot(omegalist_bckwd,OscAmpl_list_bckwd,'bo--','markersize',8);
xlabel('Forcing frequency \omega');
ylabel('Steady Oscillation Amplitude')
legend('forward frequency sweep', 'backward frequency sweep')
title({'Steady response amplitude vs frq. with the points connected',['cubic nonlinearity alpha = ' num2str(param.alpha)]});

save amplitudedata_fwdback25