% this program finds a fixed point for the Poincare Map, thereby finding
% an initial condition that is on the limit cycle and the Poincare Section

% as we know, this method is better than just finding the steady response amplitude
% by simulation, because this method can find both stable and unstable
% limit cycles, depending on the initial guess

% We also find the stability of the limit cycle here ...

clc; clear; close all;

Numomega = 17;

omegaMin = 0.1;
omegaMax = 3.0;
omegalist = linspace(omegaMin,omegaMax,Numomega);

amplitudeList = zeros(size(omegalist));

stableOrNotStore = [];
omegaStore = [];
amplitudeStore = [];

YResultStore = cell(2,1); temp = 0;

% equation of motion is
% xdotdot + + epsilon*c*xdot + k*x + epsilon*alpha*x^3  = A*cos(omega*t)

% hardening nonlinearity
param.c = 0.1; param.k = 1; param.alpha = 1; param.epsilon = 0.1;
param.A = 1;

% softening nonlinearity
% param.c = 0.2; param.k = 1; 
% param.alpha = -4;  % cubic nonlinearity, softening
% param.epsilon = 0.1; % small parameter that keeps damping and nonlinearity small
% param.A = 0.02; % forcing amplitude

NumRandomSeeds = 10;

for i = 1:Numomega
    
    for j = 1:NumRandomSeeds
        param.omega = omegalist(i); % forcing frequency for the Duffing oscillator
        
        Y0 = [0.3*randn; 0.3*randn;];
        Y0 = PoincareMap_ForcedDuffing(Y0,param); 
        % just run for one cycle just so that the initial condition is not too crazy
        
        % Using the root find to solve for a fixed point of the Poincare map and
        % therefor an initial condition for the limit cycle, assuming fsolve
        % converges
        display('Using fsolve to find the fixed point of the Poincare map and therefore an initial condition on the limit cycle');
        options = optimset('display','iter','MaxIter',30);
        Yresult = fsolve(@differenceYiYiplus1,Y0,options,param);
        howgoodistheresult = differenceYiYiplus1(Yresult,param);
        
        if norm(howgoodistheresult)<1e-5 % successfully found a periodic motion
            
            
            %% find amplitude by simulating again
            amplitudeList(i) = findamplitude_ForcedDuffing(Yresult,param);
            
            %% We now compute the Jacobian of the Poincare map so we can examine the
            % stability of the limit cycle
            h = 1e-5; % because the ODEs give about 1e-9 accuracy. optimal finite difference step size
            J = numerical_Jacobian(@PoincareMap_ForcedDuffing,Yresult,h,param)
            eigVals = eig(J)
            % checking if all the eigenvalues are less than one in absolute value
            if all(abs(eigVals)<1)
                %         display('All eigenvalues are less than 1 in absolute value, hence stable limit cycle');
                stableornot = 1;
                
                stableOrNotStore = [stableOrNotStore; stableornot];
                omegaStore = [omegaStore; omegalist(i)];
                amplitudeStore = [amplitudeStore; amplitudeList(i)];
                
                temp = temp+1;
                YResultStore{temp} = Yresult;
                
                display('This is a stable limit cycle, because all eigenvalues have absolute values less than 1');
                figure(3); hold on;
                plot(omegalist(i),amplitudeList(i),'bo');
                
            else
                %         display('At least one eigenvalue is greater than or equal to 1 in absolute value');
                stableornot = 0;
                
                stableOrNotStore = [stableOrNotStore; stableornot];
                omegaStore = [omegaStore; omegalist(i)];
                amplitudeStore = [amplitudeStore; amplitudeList(i)];
                
                temp = temp+1;
                YResultStore{temp} = Yresult;
                
                display('This is an unstable limit cycle, because NOT all eigenvalues have absolute values less than 1');
                figure(3); hold on;
                plot(omegalist(i),amplitudeList(i),'ro');
                
            end
            
        else
            amplitudeList(i) = NaN; % setting NaN amplitude when no periodic motion was found
            
                stableOrNotStore = [stableOrNotStore; NaN];
                omegaStore = [omegaStore; omegalist(i)];
                amplitudeStore = [amplitudeStore; NaN];
                
                temp = temp+1;
                YResultStore{temp} = NaN;
            
        end
        
    end
    
    
    
end

%% all the unstable periodic motions
omegaUnstable = omegaStore(find(stableOrNotStore==0))
YResultUnstable = YResultStore(find(stableOrNotStore==0))