% this program simulates the sinusoidally forced Duffing oscillator for some time T, 
% for a given initial condition

% starting with a clean slate
clc; clear all; close all;

% time duration and time list
tmax = 400; Numtpoints = 10000;
tspan = linspace(0,tmax,Numtpoints)'; % equally spaced time output
options = odeset('reltol',1e-6,'abstol',1e-6); % ode options

% system parameters
param.c = 0.1; param.k = 2; param.epsilon = 0.4;
param.omegaforce = 2; % forcing frequency
param.Aforce = 5; % forcing amplitude

% param.c = 0.1; param.k = 1; param.epsilon = 0.4;
% param.Aforce = 1*param.epsilon; 
% param.omegaforce = 1;

% initial condition
Y0 = [0.3; 0];
[tlist,Ylist] = ode15s(@OdeForcedDuffing,tspan,Y0,options,param);

% % % just picking the second half of 
% tlist = tlist(round(Numtpoints/2):end); tlist = tlist-tlist(1);
% Ylist = Ylist(round(Numtpoints/2):end,:);
% Numtpoints = length(tlist);

% plot in phase plane
figure(1);
plot(Ylist(:,1),Ylist(:,2)); xlabel('x'); ylabel('xdot');
axis equal;

% plot x and xdot as functions of time
figure(2);
subplot(211); plot(tlist,Ylist(:,1)); xlabel('t'); ylabel('x(t)'); hold on;
subplot(212); plot(tlist,Ylist(:,2)); xlabel('t'); ylabel('xdot(t)'); hold on;

% (fast) fourier transform of the input and the output
% I just copied the code below from MATLAB's online example for fft (try: doc fft)
figure(3);
forcingfunction = param.Aforce*cos(param.omegaforce*tlist);
NFFT = 2^nextpow2(length(tlist)); % Next power of 2 from length of y
fft_forcing = fft(forcingfunction,NFFT)/length(tlist);
Fs = length(tlist)/tmax;  % sampling frequency
f = (2*pi)*Fs/2*linspace(0,1,NFFT/2+1);
% Plot single-sided amplitude spectrum.
zf  = 2*abs(fft_forcing(1:NFFT/2+1));
plot(f,zf);
title('Fourier Spectrum of External Forcing function');
xlabel('Omega (Hz) -- in angular frequency');
axis([0 10*param.omegaforce 0 max(zf)]);


figure(4);
NFFT = 2^nextpow2(length(tlist)); % Next power of 2 from length of y
fft_response = fft(Ylist(:,1),NFFT)/length(tlist);
Fs = length(tlist)/tmax;  % sampling frequency
f = (2*pi)*Fs/2*linspace(0,1,NFFT/2+1);
% Plot single-sided amplitude spectrum.
zf  = 2*abs(fft_response(1:NFFT/2+1));
plot(f,zf);
title('Fourier Spectrum of the oscillation Response');
xlabel('in angular frequency (Hz)');
axis([0 10*param.omegaforce 0 max(zf)]);

% the key observation is that even though there is only a single peak in
% the forcing function, the oscillation response has multiple peaks. This
% is a salient feature of forced nonlinear systems



% 
% figure(5); % forced undamped situation, we can derive analytical solution by regular perturbation ... let's compare with that.
% % let us resimulate for about one period 
% Y0 = [max(Ylist(15000:end,1)); 0]'; % starting from the steady state of the original oscillation
% tspan = linspace(0,2*pi/param.omegaforce,100)';
% [tlist,ylist] = ode45(@OdeForcedDuffing,tspan,Y0,options,param);
% plot(tlist,ylist(:,1),'r');
% omega0 = sqrt(param.k);
% y0_approx = param.Aforce/(omega0^2-param.omegaforce^2)*cos(param.omegaforce*tlist);
% y1_approx = y0_approx-param.epsilon*(3/4*param.Aforce^3/(omega0^2-param.omegaforce^2)^4*cos(param.omegaforce*tlist) ...
%     + 1/4*param.Aforce^3/(omega0^2-param.omegaforce^2)^3/(omega0^2-9*param.omegaforce^2)*cos(3*param.omegaforce*tlist));
% hold on; plot(tlist,y0_approx,'b'); 
% hold on; plot(tlist,y1_approx,'k');
% legend('correct numerical solution','first approximation','second approximation');
