% this program simulates the sinusoidally forced Duffing oscillator for some time T, 
% for a given initial condition

% Most significantly, for a single value of all the parameters, FIVE
% different stable periodic motions coexist. Which one you find yourself on
% depends on what your initial condition is. Each of the 5 cycles has its
% own periodic orbit. From Ueda 1980, as cited in "Nonlinear Dynamics and
% Chaos" by Thomson and Stewart.

% starting with a clean slate
clc; clear all;  % close all;

% time duration and time list
tmax = 100;
tspan = linspace(0,tmax,500)'; % equally spaced time output

% solving the ODE
options = odeset('reltol',1e-10,'abstol',1e-10);

param.c = 0.08; param.k = 0; param.epsilon = 1;

param.omegaforce = 1; % forcing frequency
param.Aforce = 0.2; % forcing amplitude

% various initial conditions lead to various periodic orbits
% Try running the program with each of these initial conditions and the
% system converges to different stable periodic orbit!!
% Y0 = [-0.21; 0.02]; % initial condition 1
% Y0 = [1.05; 0.77];  % initial condition 2
% Y0 = [-0.67; 0.02]; % initial condition 3
% Y0 = [-0.46; 0.30]; % initial condition 4
% Y0 = [-0.43; 0.12]; % initial condition 5

[tlist,Ylist] = ode45(@OdeForcedDuffing,tspan,Y0,options,param);

% plot in 2D
figure(1); 
plot(Ylist(:,1),Ylist(:,2)); xlabel('x'); ylabel('xdot');
axis equal;

% plot x and xdot as functions of time
figure(2);
subplot(211); plot(tlist,Ylist(:,1)); xlabel('t'); ylabel('x(t)'); hold on;
subplot(212); plot(tlist,Ylist(:,2)); xlabel('t'); ylabel('xdot(t)'); hold on;

figure(3); hold on; 
plot3(tlist,Ylist(:,1),Ylist(:,2),'k'); xlabel('t'); ylabel('x'); zlabel('xdot');
title('Phase space in 3D');


